import java.util.List;
import java.util.ArrayList;

import net.canarymod.Canary;
import net.canarymod.commandsys.*;
import net.canarymod.plugin.Plugin;
import net.canarymod.logger.Logman;
import net.canarymod.LineTracer;
import net.canarymod.chat.MessageReceiver;
import net.canarymod.api.world.World;
import net.canarymod.api.world.blocks.BlockType;
import net.canarymod.api.world.blocks.Sign;
import net.canarymod.api.world.blocks.Block;
import net.canarymod.api.world.blocks.properties.helpers.StandingSignProperties;
import net.canarymod.api.world.blocks.properties.helpers.BlockProperties.Rotation;
import net.canarymod.api.world.position.Location;
import net.canarymod.api.world.position.Direction;
import net.canarymod.api.entity.Entity;
import net.canarymod.api.entity.living.humanoid.Player;
import net.canarymod.api.chat.ChatComponent;

public class SchilderPlugin extends Plugin implements CommandListener {
    private List<Block> schilder;
    private List<String> besitzer;

    @Override
    public boolean enable() {
        try{
            Canary.commands().registerCommands(this, this, false);
            schilder = new ArrayList<Block>();
            besitzer = new ArrayList<String>();
        }
        catch(Exception e){
            getLogman().error("Befehl konnte nicht registriert werden");
        }
        return true;
    }

    @Override
    public void disable() {
    }

    @Command(aliases = {"schild"},
             description = "Platziert eine Schild oder ndert die Beschriftung eines Schildes",
             permissions = {""},
             toolTip = "/schild <bauen|text> (<id> <zeile> <beschriftung>)",
             min = 2)
    public void schildBefehl(MessageReceiver caller, String[] parameter){ 
        if(caller instanceof Player){
            Player spieler = (Player) caller; 
            World welt = spieler.getWorld();
            
            if(parameter[1].equals("bauen")){
                LineTracer tracer = new LineTracer(spieler);
                Block b = tracer.getTargetBlock();
                
                if(b != null){
                    Location position = b.getLocation();  
                    position.setY(position.getY()+1);
                    
                    Block schild = welt.getBlockAt(position); 
                    schild.setType(BlockType.StandingSign);
                    StandingSignProperties.applyRotation(schild, directionZuUmgekehrterRotation(spieler.getCardinalDirection()));
                    schild.update();
                    
                    schilder.add(schild);
                    besitzer.add(spieler.getUUID().toString());
                    spieler.message("Das neu gebaute Schild hat die ID " + (schilder.size()-1));
                } 
                else{
                    spieler.message("Kein gltiges Ziel anvisiert.");
                }
            }
            else if(parameter[1].equals("text")){
                int schildNummer = Integer.valueOf(parameter[2]);
                int zeile = Integer.valueOf(parameter[3]);
            
                if((schildNummer >= schilder.size()) || (schildNummer < 0)){
                    spieler.message("Ungltige SchildID");
                }
                else{
                    String text = "";
                    for(int i = 4; i < parameter.length; i++){
                        if(i > 4)
                            text += " ";
                        text += parameter[i];
                    }
            
                    ChatComponent c = Canary.factory().getChatComponentFactory().newChatComponent(text);
                    Sign schild = (Sign) welt.getTileEntity(schilder.get(schildNummer));
            
                    if(schild == null){
                        spieler.message("Dieses Schild wurde leider zerstrt.");
                    }
                    else{
                        if(spieler.getUUID().toString().equals(besitzer.get(schildNummer))){
                            schild.setComponentOnLine(c, zeile);
                            schild.update();
                        }
                        else{
                            spieler.message("Dieses Schild gehrt einem anderen Spieler.");
                        }
                    }
                }
            }
            else{
                spieler.message("Falscher Parameter."); 
            }
        }
        else{
            getLogman().info("Dieser Befehl kann nur von Spielern ausgefhrt werden.");
        }
    }

    private Rotation directionZuUmgekehrterRotation(Direction d){
        if(d == Direction.NORTH)
            return Rotation.SOUTH;
        else if(d == Direction.NORTHEAST)
            return Rotation.SOUTHWEST;
        else if(d == Direction.EAST)
            return Rotation.WEST;
        else if(d == Direction.SOUTHEAST)
            return Rotation.NORTHWEST;
        else if(d == Direction.SOUTH)
            return Rotation.NORTH;
        else if(d == Direction.SOUTHWEST)
            return Rotation.NORTHEAST;
        else if(d == Direction.WEST)
            return Rotation.EAST;
        else
            return Rotation.SOUTHEAST;
    }
}