import java.util.List;
import java.util.ArrayList;
import java.util.HashMap;

import net.canarymod.Canary;
import net.canarymod.commandsys.*;
import net.canarymod.plugin.Plugin;
import net.canarymod.plugin.PluginListener;
import net.canarymod.logger.Logman;
import net.canarymod.LineTracer;
import net.canarymod.chat.MessageReceiver;
import net.canarymod.api.world.World;
import net.canarymod.api.world.blocks.BlockType;
import net.canarymod.api.world.blocks.Sign;
import net.canarymod.api.world.blocks.Block;
import net.canarymod.api.world.blocks.properties.helpers.StandingSignProperties;
import net.canarymod.api.world.blocks.properties.helpers.BlockProperties.Rotation;
import net.canarymod.api.world.position.Location;
import net.canarymod.api.world.position.Direction;
import net.canarymod.api.entity.Entity;
import net.canarymod.api.entity.living.humanoid.Player;
import net.canarymod.api.chat.ChatComponent;
import net.canarymod.database.Database;
import net.canarymod.database.exceptions.DatabaseWriteException;
import net.canarymod.database.exceptions.DatabaseReadException;
import net.canarymod.hook.HookHandler;
import net.canarymod.hook.system.LoadWorldHook;

public class SchilderPlugin extends Plugin implements CommandListener, PluginListener {
    private List<Block> schilder;
    private List<String> besitzer;

    @Override
    public boolean enable() {
        try{
            Canary.commands().registerCommands(this, this, false);
            Canary.hooks().registerListener(this, this);
            schilder = new ArrayList<Block>();
            besitzer = new ArrayList<String>();
        }
        catch(Exception e){
            getLogman().error("Befehl konnte nicht registriert werden");
        }
        return true;
    }

    @HookHandler
    public void weltGeladen(LoadWorldHook ereignis){
        SchilderTabelle schilderTabelle = new SchilderTabelle(-1,"",null);
        HashMap<String,Object> filter = new HashMap<String, Object>();

        besitzer.clear();
        schilder.clear();

        boolean wiederholen = true;
        int i = 0;

        while(wiederholen){
            filter.clear();
            filter.put("nummer", i);

            try{
                Database.get().load(schilderTabelle, filter);
            }
            catch(DatabaseReadException e){
                getLogman().error("Fehler beim Lesen");
            }

            if(schilderTabelle.nummer > -1){
                besitzer.add(schilderTabelle.besitzer);
                schilder.add(schilderTabelle.getSchild(ereignis.getWorld()));
            }
            else{
                wiederholen = false;
            }

            schilderTabelle = new SchilderTabelle(-1,"",null);
            i++;
        }
    }

    @Override
    public void disable() {
        HashMap<String,Object> filter = new HashMap<String, Object>();

        try{
            Database.get().removeAll(new SchilderTabelle(-1,"",null), filter);
        }
        catch(DatabaseWriteException e){
            getLogman().error("Fehler beim Lschen der Daten");
        }

        for(int i = 0; i < schilder.size(); i++){
            SchilderTabelle eintrag = new SchilderTabelle(i, besitzer.get(i), schilder.get(i));
            try{
                Database.get().insert(eintrag);
            }
            catch(DatabaseWriteException e){
                getLogman().error("Fehler beim Speichern der Daten");
            }
        }
    }

    @Command(aliases = {"schild"},
             description = "Platziert eine Schild oder ndert die Beschriftung eines Schildes",
             permissions = {""},
             toolTip = "/schild <bauen|text> (<id> <zeile> <beschriftung>)",
             min = 2)
    public void schildBefehl(MessageReceiver caller, String[] parameter){ 
        if(caller instanceof Player){
            Player spieler = (Player) caller; 
            World welt = spieler.getWorld();
            
            if(parameter[1].equals("bauen")){
                LineTracer tracer = new LineTracer(spieler);
                Block b = tracer.getTargetBlock();
                
                if(b != null){
                    Location position = b.getLocation();  
                    position.setY(position.getY()+1);
                    
                    Block schild = welt.getBlockAt(position); 
                    schild.setType(BlockType.StandingSign);
                    StandingSignProperties.applyRotation(schild, directionZuUmgekehrterRotation(spieler.getCardinalDirection()));
                    schild.update();
                    
                    schilder.add(schild);
                    besitzer.add(spieler.getUUID().toString());
                    spieler.message("Das neu gebaute Schild hat die ID " + (schilder.size()-1));
                } 
                else{
                    spieler.message("Kein gltiges Ziel anvisiert.");
                }
            }
            else if(parameter[1].equals("text")){
                int schildNummer = Integer.valueOf(parameter[2]);
                int zeile = Integer.valueOf(parameter[3]);
            
                if((schildNummer >= schilder.size()) || (schildNummer < 0)){
                    spieler.message("Ungltige SchildID");
                }
                else{
                    String text = "";
                    for(int i = 4; i < parameter.length; i++){
                        if(i > 4)
                            text += " ";
                        text += parameter[i];
                    }
            
                    ChatComponent c = Canary.factory().getChatComponentFactory().newChatComponent(text);
                    Sign schild = (Sign) welt.getTileEntity(schilder.get(schildNummer));
            
                    if(schild == null){
                        spieler.message("Dieses Schild wurde leider zerstrt.");
                    }
                    else{
                        if(spieler.getUUID().toString().equals(besitzer.get(schildNummer))){
                            schild.setComponentOnLine(c, zeile);
                            schild.update();
                        }
                        else{
                            spieler.message("Dieses Schild gehrt einem anderen Spieler.");
                        }
                    }
                }
            }
            else{
                spieler.message("Falscher Parameter."); 
            }
        }
        else{
            getLogman().info("Dieser Befehl kann nur von Spielern ausgefhrt werden.");
        }
    }

    private Rotation directionZuUmgekehrterRotation(Direction d){
        if(d.equals(Direction.NORTH))
            return Rotation.SOUTH;
        else if(d.equals(Direction.NORTHEAST))
            return Rotation.SOUTHWEST;
        else if(d.equals(Direction.EAST))
            return Rotation.WEST;
        else if(d.equals(Direction.SOUTHEAST))
            return Rotation.NORTHWEST;
        else if(d.equals(Direction.SOUTH))
            return Rotation.NORTH;
        else if(d.equals(Direction.SOUTHWEST))
            return Rotation.NORTHEAST;
        else if(d.equals(Direction.WEST))
            return Rotation.EAST;
        else
            return Rotation.SOUTHEAST;
    }
}