import java.util.Map;
import java.util.HashMap;
import java.util.TreeMap; 
import java.util.Comparator; 

import net.canarymod.Canary;
import net.canarymod.plugin.Plugin;
import net.canarymod.plugin.PluginListener;
import net.canarymod.logger.Logman;
import net.canarymod.commandsys.Command;
import net.canarymod.commandsys.CommandListener;
import net.canarymod.chat.MessageReceiver;

import net.canarymod.hook.HookHandler;
import net.canarymod.hook.Hook;
import net.canarymod.hook.player.PlayerRespawnedHook;
import net.canarymod.hook.player.ItemUseHook;
import net.canarymod.hook.player.ConnectionHook;
import net.canarymod.hook.entity.ProjectileHitHook;

import net.canarymod.api.entity.living.humanoid.Player;
import net.canarymod.api.entity.throwable.Snowball;
import net.canarymod.api.entity.EntityType;

import net.canarymod.api.inventory.PlayerInventory;
import net.canarymod.api.inventory.ItemType;
import net.canarymod.api.inventory.Item;

import net.canarymod.database.Database;
import net.canarymod.database.exceptions.DatabaseWriteException;
import net.canarymod.database.exceptions.DatabaseReadException;


public class Schneeballschlacht extends Plugin implements PluginListener, CommandListener {
    private HashMap<String,Integer> highscore;

    @Override
    public boolean enable() {
        highscore = new HashMap<String, Integer>();
      
        HighscoreTabelle highscoreTabelle = new HighscoreTabelle();
        HashMap<String,Object> filter = new HashMap<String, Object>();
        boolean wiederholen = true;
        int i = 0;

        while(wiederholen){
            filter.clear();
            filter.put("nummer", i);

            try{
                Database.get().load(highscoreTabelle, filter);
            }
            catch(DatabaseReadException e){
                getLogman().error("Fehler beim Lesen");
            }

            if(highscoreTabelle.nummer > -1){
                highscore.put(highscoreTabelle.spieler, highscoreTabelle.punkte);
            }
            else{
                wiederholen = false;
            }

            highscoreTabelle = new HighscoreTabelle();
            i++;
        }
      
        Canary.hooks().registerListener(this, this);
        try{
            Canary.commands().registerCommands(this, this, false);
        }
        catch(Exception e){
            getLogman().error("Befehl konnte nicht registriert werden");
        }
        return true;     
    }
    
    @Override
    public void disable() {
        HashMap<String,Object> filter = new HashMap<String, Object>();

        try{
            Database.get().removeAll(new HighscoreTabelle(), filter);
        }
        catch(DatabaseWriteException e){
            getLogman().error("Fehler beim Lschen der Daten");
        }
        
        String[] spielerListe = highscore.keySet().toArray(new String[highscore.size()]);
        
        for(int i = 0; i < spielerListe.length; i++){
            HighscoreTabelle eintrag = new HighscoreTabelle();
            eintrag.nummer = i;
            eintrag.spieler = spielerListe[i];
            eintrag.punkte = highscore.get(spielerListe[i]);
            
            try{
                Database.get().insert(eintrag);
            }
            catch(DatabaseWriteException e){
                getLogman().error("Fehler beim Speichern der Daten");
            }
        }
    }
    
    private void schneebaelleGeben(Player spieler){
        PlayerInventory inventar = spieler.getInventory();
      
        if(inventar.hasItem(ItemType.SnowBall)){
            //Spieler hat bereits mindestens einen Schneeball
            inventar.getItem(ItemType.SnowBall).setAmount(16);        
        }
        else{
            //Spieler hat noch keinen Schneeball     
            if(inventar.getEmptySlot() < 0 || inventar.getEmptySlot() > 35){
                //Inventar voll
                inventar.removeItem(inventar.getSlot(0));
            }         
            inventar.addItem(ItemType.SnowBall, 16);  
        }
    }
    
    @HookHandler
    public void respawn(PlayerRespawnedHook ereignis){
        schneebaelleGeben(ereignis.getPlayer());
    }
    
    @HookHandler
    public void spielerBetrittServer(ConnectionHook ereignis){
        schneebaelleGeben(ereignis.getPlayer());            
    }
    
    @HookHandler
    public void schneeballGeworfen(ItemUseHook ereignis){
        Player spieler = ereignis.getPlayer();
        Item gegenstand = ereignis.getItem();
      
        if(gegenstand.getType() == ItemType.SnowBall){
            //Schneeball geworfen
            gegenstand.setAmount(gegenstand.getAmount()+1);
        }
    }
    
    @HookHandler
    public void treffer(ProjectileHitHook ereignis){
        if(ereignis.getEntityHit() != null && ereignis.getEntityHit().isPlayer()){
            //Spieler wurde getroffen
            if(ereignis.getProjectile().getEntityType() == EntityType.SNOWBALL){
                //geworfenes Objekt ist Schneeball
                Snowball schneeball = (Snowball) ereignis.getProjectile();
                
                Player werfer = (Player) schneeball.getThrower();
                Player ziel = (Player) ereignis.getEntityHit();
          
                String werferId = werfer.getUUID().toString();
                String zielId = ziel.getUUID().toString();
                
                if(highscore.containsKey(werferId)){
                    highscore.put(werferId, highscore.get(werferId)+1);  
                }
                else{
                    highscore.put(werferId, 1);
                }
                
                if(highscore.containsKey(zielId)){
                    highscore.put(zielId, highscore.get(zielId)-1);
                }
                else{
                    highscore.put(zielId, -1);
                }
          
                werfer.message("Dein neuer Punktestand ist " + highscore.get(werferId));
                ziel.message("Dein neuer Punktestand ist " + highscore.get(zielId));
            }
        }
    }
    
    @Command(aliases = {"highscore"},
             description = "Zeigt die Highscore-Liste an",
             permissions = {""},
             toolTip = "/highscore")
    public void highscoreAnzeigen(MessageReceiver caller, String[] parameter){
        Vergleich bvc =  new Vergleich(highscore);
        TreeMap<String, Integer> map = new TreeMap<String, Integer>(bvc);
        map.putAll(highscore);
        
        String[] spielerListe = map.keySet().toArray(new String[map.size()]);
        
        for(int i = 0; i < spielerListe.length; i++){
            if(Canary.getServer().getPlayerFromUUID(spielerListe[i]) != null){
            String spielerName = Canary.getServer().getPlayerFromUUID(spielerListe[i]).getDisplayName();
            caller.message((i+1)+ ": " + spielerName + " (" + highscore.get(spielerListe[i]) + ")");
            }
        }
    }
}